import React, { useState, useEffect } from 'react';
import { render } from 'react-dom';
import { printSchema, buildClientSchema, getIntrospectionQuery, parse } from 'graphql';
import TykGraphiQL from '@tyk-technologies/graphiql';

const changeProtocol = (url) => url.match(/http:\/\//) ? url.replace(/http:\/\//, 'ws://') :  url.replace(/https:\/\//, 'wss://');

const GraphqlIDE = ({
  selectedApi,
  setSelectedApi,
  config,
  headers,
  setHeaders,
}) => {
  let subscriptionsClient;
  const wsUrl = changeProtocol(selectedApi.url);
  const [schema, setSchema] = useState(config.schema);
  const [subscriptionActivated, setSubscriptionActivated] = useState(false);

  const fetcher = async (graphQLParams) => {
    const data = await fetch(selectedApi.url, {
      method: 'POST',
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json',
        ...headers,
      },
      body: JSON.stringify(graphQLParams),
      credentials: 'same-origin',
    });
    return data.json().catch(() => data.text());
  };

  const menu = {
    label: 'Select Api',
    title: 'Select Api',
    onSelect: (item) => {
      setSelectedApi(item.value);
    },
    items: config.schemaUrls.map((api) => ({
      label: api.name,
      value: {
        url: api.url,
      },
    })),
  };

  const getSchema = () => {
    return fetcher({
      query: getIntrospectionQuery(),
    }).then((result) => {
      if (result && result.error) {
        return;
      }
      const schema = printSchema(buildClientSchema(result.data));
      setSchema(schema);
    });
  };

  useEffect(() => {
    getSchema();
  }, [selectedApi]);

  const handleHeaderEdit = (headerStr) => {
    try {
      const tempHeaders = JSON.parse(headerStr);
      setHeaders(tempHeaders);
    } catch (error) {
      return;
    }
  };

  // Hybrid fetcher : if operation is subscription use subscriptionsClient else uses normal fetcher.
  const hybridFetcher = (params, opts) => {
    const parsedQuery = parse(params.query);
    const isSubscriptionOperation = parsedQuery.definitions.some((def) => def.operation === 'subscription');
    if (subscriptionActivated && subscriptionsClient) {
      subscriptionsClient.unsubscribeAll()
      setSubscriptionActivated(false);
    }
    if (isSubscriptionOperation) {
      const subscriptionUrl = `${wsUrl}?${opts?.headers ? new URLSearchParams(opts?.headers) : ''}`
      subscriptionsClient = new window.SubscriptionsTransportWs.SubscriptionClient(subscriptionUrl, { reconnect: false });
      subscriptionsClient.client.onerror = () => config.onSubscriptionError();
      setSubscriptionActivated(true);
      return subscriptionsClient.request(params);
    }
    return fetcher(params);
  };

  return (
    <TykGraphiQL
      fetcher={hybridFetcher}
      schema={schema}
      ideName={"Tyk"}
      menu={menu.items.length > 1 && menu}
      onEditHeaders={handleHeaderEdit}
      headerEditorEnabled
      shouldPersistHeaders={false}
    />
  );
};

export const TykGraphqlIDE = ({ config }) => {

  const [headers, setHeaders] = useState({});

  const [selectedApi, setSelectedApi] = useState({
    name: config?.schemaUrls[0]?.name,
    url: config?.schemaUrls[0]?.url,
  });

  return (
    <GraphqlIDE
      selectedApi={selectedApi}
      setSelectedApi={setSelectedApi}
      headers={headers}
      setHeaders={setHeaders}
      config={config}
    />
  );
};

class GraphiqlInit {
  constructor(config) {
    const { containerId } = config;
    render(
      <TykGraphqlIDE config={config} />,
      document.getElementById(containerId)
    );
  }
}
window.GraphiqlInit = GraphiqlInit;
